@@ -5,6 +5,7 @@ Created on 2023-10-3
 """
 
 import datetime
+import logging
 import sys
 import traceback
 from dataclasses import dataclass, field
@@ -23,7 +24,7 @@ class GridConfig:
     column_defs: Optional[List[Dict]] = None
     options: Dict = field(default_factory=dict)
     # optics
-    theme: str = "ag-theme-material"
+    theme: str = "material"
     classes: str = "h-screen overflow-auto"
     all_cols_html: bool = True
     # behavior
@@ -39,6 +40,9 @@ class GridConfig:
     auto_size_columns: bool = True
     # buttons
     with_buttons: bool = False
+    button_names: List[str] = field(
+        default_factory=lambda: ["new", "delete", "all", "fit"]
+    )
     prepend_new: bool = True
     html_columns: List[int] = field(default_factory=list)
     keygen_callback: Optional[Callable] = None
@@ -67,9 +71,10 @@ class ListOfDictsGrid:
         self.lod = lod
         self.config = config or GridConfig()
         self.lod_index = {}
+        self.all_selected = False  # track selection state
         try:
             if self.config.with_buttons:
-                self.setup_button_row()
+                self.setup_button_row(self.config.button_names)
             # Update options to include onGridReady event handling
             self.config.options[":onGridReady"] = (
                 "(params) => params.columnApi.autoSizeAllColumns()"
@@ -189,40 +194,51 @@ class ListOfDictsGrid:
         Raises:
             Exception: Re-raises the exception in debug mode for further debugging.
         """
+        error_msg = f"Exception caught in ListOfDictsGrid: {str(ex)}"
         if self.config.debug:
             # Print a stack trace to stderr
-            print("Exception caught in ListOfDictsGrid:", file=sys.stderr)
+            print(error_msg, file=sys.stderr)
             traceback.print_exc()
-            # Optionally, re-raise the exception for further debugging.
-            raise ex
-        elif self.config.exception_callback:
+        if self.config.exception_callback:
             self.config.exception_callback(ex)
         else:
-            # If not in debug mode, notify the user with a general error message.
-            # Ensure that ui.notify or a similar method is available and properly configured.
-            ui.notify(
-                f"Unhandled exception {str(ex)} occurred in ListOfDictsGrid",
-                type="error",
-            )
+            logging.error(error_msg, exc_info=ex)
+            pass
+
+    def get_index(self, lenient: bool = False, lod=None, key_col: str = None):
+        """
+        get and index (dict of dicts) of the given list of
+        dicts based on the given key column - if no key_col is given
+        index by row number starting from 1
+        """
+        lod_index = {}
+        if lod is None:
+            lod = self.lod
+        if lod:
+            for row_index, row in enumerate(lod):
+                if key_col is None:
+                    lod_index[row_index + 1] = row
+                else:
+                    if self.config.key_col in row:
+                        key_value = row[key_col]
+                        lod_index[key_value] = row
+                    else:
+                        msg = f"missing key column {self.config.key_col} in row {row_index}"
+                        if not lenient:
+                            raise Exception(msg)
+                        else:
+                            print(msg, file=sys.stderr)
+                        # missing key
+                        pass
+        return lod_index
 
     def update_index(self, lenient: bool = False):
         """
-        update the index based on the given key column
+        update my index (with view records)
         """
-        self.lod_index = {}
-        if self.lod:
-            for row_index, row in enumerate(self.lod):
-                if self.config.key_col in row:
-                    key_value = row[self.config.key_col]
-                    self.lod_index[key_value] = row
-                else:
-                    msg = f"missing key column {self.config.key_col} in row {row_index}"
-                    if not lenient:
-                        raise Exception(msg)
-                    else:
-                        print(msg, file=sys.stderr)
-                    # missing key
-                    pass
+        self.lod_index = self.get_index(
+            lenient=lenient, lod=self.lod, key_col=self.config.key_col
+        )
 
     def get_row_for_key(self, key_value: str):
         """
@@ -301,7 +317,7 @@ class ListOfDictsGrid:
 
     def sizeColumnsToFit(self):
         if self.ag_grid:
-            self.ag_grid.run_column_method("autoSizeAllColumns")
+            self.ag_grid.run_grid_method("autoSizeAllColumns")
             self.ag_grid.update()
 
     def setDefaultColDef(self):
@@ -369,12 +385,72 @@ class ListOfDictsGrid:
         selected_rows = await self.ag_grid.get_selected_rows()
         return selected_rows
 
+    def select_rows_by_keys(self, key_values: List[Any]) -> None:
+        """
+        Select rows in the grid based on a list of key values.
+
+        Args:
+            key_values (List[Any]): List of key values to be selected.
+        """
+        row_data = self.get_row_data()
+        for row_index, row in enumerate(row_data):
+            key_value = row.get(self.config.key_col)
+            if key_value in key_values:
+                self.ag_grid.run_row_method(row_index, "setSelected", True)
+
+    async def get_selected_lod(self, lod_index=None):
+        """
+        selected rows are in view (e.g. potentially  html) format
+        get back the original list of dict rows
+        """
+        selected_lod = []
+        selected_rows = await self.get_selected_rows()
+        if lod_index is None:
+            lod_index = self.get_index(lenient=self.config.lenient, lod=self.lod)
+        for row in selected_rows:
+            key_value = row[self.config.key_col]
+            record = lod_index[key_value]
+            selected_lod.append(record)
+        return selected_lod
+
     def select_all_rows(self):
         """
         select all my ag_grid rows
         """
         self.ag_grid.run_grid_method("selectAll")
 
+    def deselect_all_rows(self):
+        """
+        deselect all my ag_grid rows
+        see https://stackoverflow.com/a/52199985/1497139
+        """
+        self.ag_grid.run_grid_method("deselectAll")
+
+    def toggle_select_all_rows(self):
+        """
+        Toggle between selecting all rows and deselecting all rows.
+        """
+        if self.all_selected:
+            self.deselect_all_rows()
+        else:
+            self.select_all_rows()
+        self.all_selected = not self.all_selected
+        self.update_select_all_toggle_button()
+
+    def update_select_all_toggle_button(self):
+        """
+        Update the toggle button's label and icon based on selection state.
+        see https://github.com/zauberzeug/nicegui/discussions/3596
+        """
+        stb = self.select_toggle_button
+        if self.all_selected:
+            stb.text = "None"
+            stb.props("icon=check_box_outline_blank")
+        else:
+            stb.text = "All"
+            stb.props("icon=select_all")
+        stb.update()
+
     async def delete_selected_rows(self, _args):
         """
         Delete the currently selected rows based on the key column.
@@ -423,18 +499,31 @@ class ListOfDictsGrid:
         except Exception as ex:
             self.handle_exception(ex)
 
-    def setup_button_row(self):
+    def setup_button_row(self, button_names: list):
         """
         set up a button row
+
+        Args:
+            button_names (list): the list of buttons to be setup
         """
-        with ui.row():
+        with ui.row() as self.button_row:
             # icons per https://fonts.google.com/icons
             if self.config.editable:
-                ui.button("New", icon="add", on_click=self.new_row)
-                ui.button("Delete", icon="delete", on_click=self.delete_selected_rows)
-            # ui.button("Fit", icon="arrow_range", on_click=self.onSizeColumnsToFit)
-            ui.button(
-                "All",
-                icon="select_all",
-                on_click=self.select_all_rows,
-            )
+                if "new" in button_names:
+                    self.new_button = ui.button(
+                        "New", icon="add", on_click=self.new_row
+                    )
+                if "delete" in button_names:
+                    self.delete_button = ui.button(
+                        "Delete", icon="delete", on_click=self.delete_selected_rows
+                    )
+            if "fit" in button_names:
+                self.fit_button = ui.button(
+                    "Fit", icon="fit_screen", on_click=self.onSizeColumnsToFit
+                )
+            if "all" in button_names:
+                self.select_toggle_button = ui.button(
+                    "All",
+                    icon="select_all",
+                    on_click=self.toggle_select_all_rows,
+                )
