@@ -4,8 +4,9 @@ Created on 2023-09-12
 @author: wf
 """
 
+from basemkit.yamlable import lod_storable
+
 import ngwidgets
-from ngwidgets.yamlable import lod_storable
 
 
 @lod_storable
@@ -17,7 +18,7 @@ class Version:
     name = "ngwidgets"
     version = ngwidgets.__version__
     date = "2023-09-10"
-    updated = "2024-08-10"
+    updated = "2025-11-07"
     description = "NiceGUI widgets"
 
     authors = "Wolfgang Fahl"
@@ -26,7 +27,7 @@ class Version:
     chat_url = "https://github.com/WolfgangFahl/nicegui_widgets/discussions"
     cm_url = "https://github.com/WolfgangFahl/nicegui_widgets"
 
-    license = f"""Copyright 2023-2024 contributors. All rights reserved.
+    license = f"""Copyright 2023-2025 contributors. All rights reserved.
 
   Licensed under the Apache License 2.0
   http://www.apache.org/licenses/LICENSE-2.0
