@@ -5,6 +5,7 @@ Created on 2023-09-10
 """
 
 import asyncio
+import logging
 import os
 import sys
 import traceback
@@ -13,11 +14,11 @@ import uuid
 from dataclasses import field
 from typing import Any, Callable, Dict, List, Optional, Union
 
+from basemkit.yamlable import lod_storable
 from nicegui import Client, core, ui
 
 from ngwidgets.color_schema import ColorSchema
 from ngwidgets.version import Version
-from ngwidgets.yamlable import lod_storable
 
 
 @lod_storable
@@ -34,6 +35,7 @@ class WebserverConfig:
     default_port: int = 9860
     version: Optional[Version] = None
     color_schema: ColorSchema = field(default_factory=ColorSchema.indigo)
+    fastapi_docs: Optional[bool] = True
     detailed_menu: bool = True
     timeout: Optional[float] = None
     storage_secret: Optional[str] = None
@@ -115,6 +117,7 @@ class NiceGuiWebserver(object):
         Constructor
         """
         self.debug = False
+        self.logger = logging.getLogger()
         self.do_trace = True
         if config is None:
             config = WebserverConfig()
@@ -137,6 +140,27 @@ class NiceGuiWebserver(object):
         solution_class = self.config.solution_class
         if not solution_class:
             raise TypeError("no solution_class configured")
+
+        action_result = await self.execute_action(
+            client, solution_class, wanted_action, *args, **kwargs
+        )
+        return action_result
+
+    async def execute_action(
+        self, client: Client, solution_class, wanted_action: Callable, *args, **kwargs
+    ):
+        """
+        Execute the specified action on the solution instance.
+
+        Args:
+            client (Client): The client making the request.
+            solution_class: The solution class to create an instance for to execute the action on.
+            wanted_action(Callable): The function to perform.
+            args, *kwargs: Additional arguments to pass to the action.
+
+        Returns:
+            The result of the action performed.
+        """
         solution_instance = solution_class(self, client)
 
         # Check if the solution_instance is an instance of solution_class or its subclass
@@ -161,7 +185,8 @@ class NiceGuiWebserver(object):
         # are done
         solution_instance.prepare_ui()
 
-        return await action(*args, **kwargs)
+        action_result = await action(*args, **kwargs)
+        return action_result
 
     @classmethod
     def optionalDebug(self, args):
@@ -212,6 +237,7 @@ class NiceGuiWebserver(object):
             host=args.host,
             port=args.port,
             show=args.client,
+            fastapi_docs=self.config.fastapi_docs,
             reload=False,
             storage_secret=self.config.storage_secret,
         )
@@ -241,6 +267,7 @@ class WebSolution:
         construct a client specific WebSolution
         """
         self.webserver = webserver
+        self.logger = webserver.logger
         self.config = self.webserver.config
         self.args = self.webserver.args
         self.client = client
@@ -288,7 +315,7 @@ class WebSolution:
         see https://fonts.google.com/icons?icon.set=Material+Icons for potential icon names
         """
         with ui.link(text=" ", target=target, new_tab=new_tab) as link_btn:
-            ui.button(name, icon=icon_name)
+            link_btn.button = ui.button(name, icon=icon_name)
         return link_btn
 
     def tool_button(
@@ -405,6 +432,17 @@ class WebSolution:
             else:
                 raise Exception(f"File does not exist: {input_str}")
 
+    async def toogle_hamburger(self):
+        """
+        toggle the hamburger menu
+        """
+        if not self.header:
+            return
+
+        self.header.toggle()
+        self.footer.toggle()
+        self.hamburger_button1.visible = not self.hamburger_button1.visible
+
     def setup_menu(self, detailed: bool = None):
         """
         set up the default menu home/settings and about
@@ -412,11 +450,23 @@ class WebSolution:
         Args:
             detailed(bool): if True add github,chat and help links
         """
+        self.header = None
         version = self.config.version
         if detailed is None:
             detailed = self.config.detailed_menu
         self.config.color_schema.apply()
+        # parent element for hamburger menu
+        with ui.element("div").classes(
+            "top-0 left-0 z-50 bg-transparent"
+        ) as self.hamburger_container:
+            self.hamburger_button1 = ui.button(
+                icon="menu", on_click=self.toogle_hamburger
+            )
+            self.hamburger_button1.visible = False
         with ui.header() as self.header:
+            self.hamburger_button = ui.button(
+                icon="menu", on_click=self.toogle_hamburger
+            )
             self.link_button("home", "/", "home", new_tab=False)
             self.link_button("settings", "/settings", "settings", new_tab=False)
             self.configure_menu()
@@ -424,7 +474,7 @@ class WebSolution:
                 self.link_button("github", version.cm_url, "bug_report")
                 self.link_button("chat", version.chat_url, "chat")
                 self.link_button("help", version.doc_url, "help")
-            self.link_button("about", "/about", "info", new_tab=False)
+                self.link_button("about", "/about", "info", new_tab=False)
 
     async def setup_footer(self):
         """
