@@ -4,7 +4,10 @@ Created on 2023-09-13
 @author: wf
 """
 
+import asyncio
+import logging
 import random
+import time
 from dataclasses import dataclass
 from datetime import datetime
 
@@ -16,12 +19,17 @@ from ngwidgets.combobox import ComboBox
 from ngwidgets.components_view import ComponentsView
 from ngwidgets.debouncer import DebouncerUI
 from ngwidgets.dict_edit import DictEdit
+from ngwidgets.gpxviewer import GPXViewer
 from ngwidgets.input_webserver import InputWebserver, InputWebSolution
+from ngwidgets.leaflet_map import LeafletMap
 from ngwidgets.lod_grid import GridConfig, ListOfDictsGrid
+from ngwidgets.log_view import LogView
 from ngwidgets.pdfviewer import pdfviewer
 from ngwidgets.progress import NiceguiProgressbar
 from ngwidgets.projects import Projects
 from ngwidgets.projects_view import ProjectsView
+from ngwidgets.sso_users_solution import SsoSolution
+from ngwidgets.task_runner import TaskRunner
 from ngwidgets.tristate import Tristate
 from ngwidgets.version import Version
 from ngwidgets.webserver import WebserverConfig
@@ -48,7 +56,7 @@ class NiceGuiWidgetsDemo(InputWebSolution):
     Demonstration Solution
     """
 
-    def __init__(self, webserver: "NiceGuiWebserver", client: Client):
+    def __init__(self, webserver: "NiceGuiWidgetsDemoWebserver", client: Client):
         """
         Initialize the NiceGuiWidgetsDemoContext.
 
@@ -61,6 +69,8 @@ class NiceGuiWidgetsDemo(InputWebSolution):
         """
         super().__init__(webserver, client)  # Call to the superclass constructor
         self.projects = self.webserver.projects
+        # pdf_url = "https://www.africau.edu/images/default/sample.pdf"
+        self.pdf_url = "https://raw.githubusercontent.com/mozilla/pdf.js/ba2edeae/web/compressed.tracemonkey-pldi-09.pdf"
 
     async def load_pdf(self):
         self.pdf_viewer.load_pdf(self.pdf_url)
@@ -108,8 +118,8 @@ class NiceGuiWidgetsDemo(InputWebSolution):
 
             # Buttons for controlling the progress bar
             with ui.row():
-                ui.button("--", on_click=lambda: update_progress(-1))
-                ui.button("++", on_click=lambda: update_progress(1))
+                ui.button("--").on_click(lambda: update_progress(-1))
+                ui.button("++").on_click(lambda: update_progress(1))
                 ui.button("Auto", on_click=toggle_auto)
 
         await self.setup_content_div(show)
@@ -127,24 +137,24 @@ class NiceGuiWidgetsDemo(InputWebSolution):
         """
 
         def foreground_no_slot(event):
-            ui.notify(f"{event.sender.text} clicked")
+            ui.notify(f"{event.sender.text} ed")
 
         async def background_no_slot(event):
             await run.io_bound(foreground_no_slot, event)
 
         def foreground_with_slot(event):
             with self.button_row:
-                ui.notify(f"{event.sender.text} clicked")
+                ui.notify(f"{event.sender.text} ed")
 
         async def background_with_slot(event):
             await run.io_bound(foreground_with_slot, event)
 
         def show():
             with ui.row() as self.button_row:
-                ui.button("foreground no slot", on_click=foreground_no_slot)
-                ui.button("background no slot", on_click=background_no_slot)
-                ui.button("foreground with slot", on_click=foreground_with_slot)
-                ui.button("background with slot", on_click=background_with_slot)
+                ui.button("foreground no slot", on_=foreground_no_slot)
+                ui.button("background no slot", on_=background_no_slot)
+                ui.button("foreground with slot", on_=foreground_with_slot)
+                ui.button("background with slot", on_=background_with_slot)
 
         await self.setup_content_div(show)
 
@@ -356,7 +366,7 @@ class NiceGuiWidgetsDemo(InputWebSolution):
 
                 # setup some grid event listeners
                 # https://www.ag-grid.com/javascript-data-grid/grid-events/
-                self.lod_grid.ag_grid.on("cellClicked", self.on_cell_clicked)
+                self.lod_grid.ag_grid.on("celled", self.on_cell_clicked)
                 self.lod_grid.ag_grid.on(
                     "cellDoubleClicked", self.on_cell_double_clicked
                 )
@@ -503,6 +513,167 @@ class NiceGuiWidgetsDemo(InputWebSolution):
 
         await self.setup_content_div(show)
 
+    async def show_log_element_handler_demo(self):
+        """
+        demonstrate logging via log_view
+        """
+
+        def on_loglevel_change(value):
+            if self.log_view:
+                self.log_view.setLevel(value)
+
+        def show():
+            with ui.row():
+                ui.select(
+                    {
+                        logging.NOTSET: "All",
+                        logging.DEBUG: f"{LogView.LOG_LEVEL_ICONS[logging.DEBUG]} Debug",
+                        logging.INFO: f"{LogView.LOG_LEVEL_ICONS[logging.INFO]} Info",
+                        logging.WARNING: f"{LogView.LOG_LEVEL_ICONS[logging.WARNING]} Warning",
+                        logging.ERROR: f"{LogView.LOG_LEVEL_ICONS[logging.ERROR]} Error",
+                        logging.CRITICAL: f"{LogView.LOG_LEVEL_ICONS[logging.CRITICAL]} Critical",
+                    },
+                    value=logging.NOTSET,
+                    on_change=lambda e: on_loglevel_change(e.value),
+                )
+
+            with ui.row():
+                ui.button("Debug", on_click=lambda: self.logger.debug("DEBUG"))
+                ui.button("Info", on_click=lambda: self.logger.info("INFO"))
+                ui.button("Warning", on_click=lambda: self.logger.warning("WARNING"))
+                ui.button("Error", on_click=lambda: self.logger.error("ERROR"))
+                ui.button("Critical", on_click=lambda: self.logger.critical("CRITICAL"))
+
+        await self.setup_content_div(show)
+
+    async def show_shell_demo(self):
+        """
+        Demonstrate Shell execution with tee and capture using safe predefined commands.
+        """
+        # predefined safe shell commands
+        commands = {
+            "Echo greeting": "echo Hello from shell",
+            "List files": "ls -l | head -5",
+            "Trigger error": "ls non_existent_file",
+            "Combined": "echo Hello from shell && ls -l | head -5 && ls non_existent_file",
+        }
+
+        # display a labeled preformatted text block
+        def labeled_pre(label: str, value: str, color: str = "black"):
+            ui.label(f"{label}:").style("font-weight: bold")
+            ui.html(f"<pre>{value}</pre>").style(f"color: {color}")
+
+        # run the selected command and display output
+        def run_cmd(cmd_select):
+            from basemkit.shell import Shell
+
+            shell = Shell()
+            cmd = commands[cmd_select.value]
+            result = shell.run(cmd, tee=True)
+            labeled_pre("CMD", cmd)
+            labeled_pre("STDOUT", result.stdout)
+            if result.stderr.strip():
+                labeled_pre("STDERR", result.stderr, color="red")
+
+        # setup the shell demo UI
+        async def show():
+            cmd_select = ui.select(
+                list(commands.keys()), label="Choose command", value="Combined"
+            ).classes("w-full")
+            ui.button("Run", on_click=lambda: run_cmd(cmd_select))
+
+        await self.setup_content_div(show)
+
+    async def show_taskrunner_demo(self):
+        """
+        Demonstrate the TaskRunner with timeout, progress, and cancellation
+        """
+        def show():
+            # Create progress bar for TaskRunner
+            progress_bar = NiceguiProgressbar(total=100, desc="Task", unit="step")
+            task_runner = TaskRunner(timeout=8.0, progress=progress_bar)
+
+            async def hint(title:str,done:bool=False,marker:str="✅"):
+                with content:
+                    content.clear()
+                with content:
+                    if not done:
+                        progress_bar.progress.visible = False
+                        progress_bar.reset()
+                        ui.spinner()
+                        marker=""
+                    ui.markdown(title+marker)
+
+            async def load_data_5secs():
+                title='Loading data ... for 5 secs'
+                await hint(title)
+
+                # Simulate work with progress updates
+                for _i in range(50):
+                    await asyncio.sleep(0.1)  # 5 seconds total
+                    progress_bar.update(2)  # 2% per step
+
+                await hint(title,True)
+
+            async def slow_load_10secs():
+                title='Loading 10 secs (this will timeout after 8 secs)...'
+                await hint(title)
+
+                # This will timeout at 8 seconds
+                for _i in range(100):
+                    await asyncio.sleep(0.1)  # 10 seconds total - will timeout
+                    progress_bar.update(1)
+
+                await hint('This should timeout!',True,marker="❌")
+
+            def blocking_task(secs:float=5):
+                time.sleep(secs)
+                return f"Blocking task  {secs} secs completed"
+
+            async def blocking_task_5secs():
+                title = 'Running blocking task for 5 secs'
+                await hint(title)
+                result = blocking_task(5)
+                await hint(result, True)
+
+            async def quick_task_1sec():
+                title='Quick task 1sec'
+                await hint(title)
+
+                await asyncio.sleep(1)  # 1 second for quick demo
+                await hint(title,True)
+
+            async def cancel_task():
+                task_runner.cancel_running()
+                task_runner.progress.reset()
+                await hint("Task canceled",True,marker='⚠️')
+
+            async def clear():
+                await hint("",True,marker='️')
+                progress_bar.progress.visible = False
+
+            with ui.card() as content:
+                ui.markdown('Ready to run tasks...')
+
+            # Show progress bar
+            progress_bar.progress.visible = True
+
+            with ui.row():
+                ui.button('Load Data (5s)', on_click=lambda: task_runner.run(load_data_5secs))
+                ui.button('Quick Task (1s)', on_click=lambda: task_runner.run(quick_task_1sec))
+                ui.button('Blocking Task (5s)', on_click=lambda: task_runner.run(blocking_task_5secs))
+                ui.button('Timeout Test (8s+)', on_click=lambda: task_runner.run(slow_load_10secs))
+                ui.button('Cancel', on_click=cancel_task)
+                ui.button('Clear', on_click=clear)
+
+            with ui.row():
+                status_label = ui.label('')
+
+                # Update status every 100ms using timer
+                ui.timer(0.1, lambda: status_label.set_text(task_runner.get_status()))
+
+        await self.setup_content_div(show)
+
     async def show_tristate_demo(self):
         """
         Demonstrate the Tristate project.
@@ -716,6 +887,96 @@ class NiceGuiWidgetsDemo(InputWebSolution):
 
         await self.setup_content_div(show)
 
+    async def show_gpxviewer(self):
+        """
+        show gpx demo
+        """
+
+        async def load_gpx(url: str):
+            viewer = GPXViewer.from_url(url)
+            viewer.show()
+
+        async def show():
+            """
+            show off GPXViewer with some samples
+            """
+
+            ui.select(
+                options=list(GPXViewer.samples.keys()),
+                label="Select GPX Track",
+                on_change=lambda e: load_gpx(GPXViewer.samples[e.value]),
+            ).classes("w-64")
+
+        await self.setup_content_div(show)
+
+    async def show_leaflet(self):
+        """Show leaflet demo with locations and path drawing"""
+
+        async def show():
+
+            self.locations = {
+                "London": (51.505, -0.090),
+                "Berlin": (52.5200, 13.4049),
+                "Hannover": (52.37487, 9.74168),
+                "New York": (40.7306, -74.0060),
+                "Beijing": (39.9042, 116.4074),
+                "Tokyo": (35.6895, 139.6917),
+            }
+            self.zoom_level = 9
+            london = self.locations["London"]
+
+            self.map = LeafletMap(
+                center=london, zoom=self.zoom_level, classes="w-full h-96"
+            )
+            await self.map.initialized()
+
+            with ui.row() as self.select_row:
+                ui.select(
+                    options=list(self.locations.keys()),  # Use city names
+                    label="Location",
+                    value="London",
+                    on_change=lambda e: setattr(
+                        self.map, "center", self.locations[e.value]
+                    ),
+                ).classes("w-40")
+                ui.select(
+                    {i: i for i in range(9, 18)},
+                    label="Zoom",
+                    value=self.zoom_level,
+                    on_change=lambda e: setattr(self.map, "zoom", e.value),
+                ).classes("w-40")
+                ui.label().bind_text_from(
+                    self.map,
+                    "center",
+                    lambda center: f"Center: {center[0]:.3f}, {center[1]:.3f}",
+                )
+                ui.label().bind_text_from(
+                    self.map, "zoom", lambda zoom: f"Zoom: {zoom}"
+                )
+                # ui.label().bind_text(lambda: f'Layers: {len(self.map.layers)}')  # Layer counter
+
+            with ui.grid(columns=2) as self.grid:
+                ui.button(
+                    icon="zoom_in",
+                    on_click=lambda: setattr(self.map, "zoom", self.map.zoom + 1),
+                )
+                ui.button(
+                    icon="zoom_out",
+                    on_click=lambda: setattr(self.map, "zoom", self.map.zoom - 1),
+                )
+                # ui.button('Fit world', on_click=lambda: self.map.run_map_method('fitWorld'))
+                ui.button("Clear layers", on_click=lambda: self.map.clear_layers())
+
+        await self.setup_content_div(show)
+
+    def configure_menu(self):
+        """
+        configure my menu
+        """
+        InputWebSolution.configure_menu(self)
+        self.sso_solution = SsoSolution(webserver=self.webserver)
+        self.sso_solution.configure_menu()
+
     async def home(self):
         """
         provide the main content page
@@ -730,12 +991,17 @@ class NiceGuiWidgetsDemo(InputWebSolution):
                 "ComboBox Demo": "/combobox",
                 "Debounce Demo": "/stars",
                 "DictEdit": "/dictedit",
+                "GPX-Viewer": "/gpxviewer",
                 "HideShow Demo": "/hideshow",
                 "Lang": "/langs",
+                "Leaflet Map": "/leaflet",
                 "ListOfDictsGrid": "/grid",
+                "Log Element Handler Demo": "/log_handler",
+                "TaskRunner Demo": "/taskrunner",
                 "Tristate Demo": "/tristate",
                 "pdfviewer": "/pdfviewer",
                 "Progressbar": "/progress",
+                "Shell Demo": "/shell",
             }
 
             # Generate the HTML using the dictionary
@@ -757,7 +1023,7 @@ class NiceGuiWidgetsDemoWebserver(InputWebserver):
 
     @classmethod
     def get_config(cls) -> WebserverConfig:
-        copy_right = "(c)2023-2024 Wolfgang Fahl"
+        copy_right = "(c)2023-2025 Wolfgang Fahl"
         config = WebserverConfig(
             short_name="ngdemo",
             timeout=6.0,
@@ -774,8 +1040,6 @@ class NiceGuiWidgetsDemoWebserver(InputWebserver):
         Constructor
         """
         InputWebserver.__init__(self, config=NiceGuiWidgetsDemoWebserver.get_config())
-        # pdf_url = "https://www.africau.edu/images/default/sample.pdf"
-        self.pdf_url = "https://raw.githubusercontent.com/mozilla/pdf.js/ba2edeae/web/compressed.tracemonkey-pldi-09.pdf"
         self.projects = Projects(topic="nicegui")
         self.projects.load()
         pass
@@ -807,10 +1071,28 @@ class NiceGuiWidgetsDemoWebserver(InputWebserver):
         async def show_progress(client: Client):
             return await self.page(client, NiceGuiWidgetsDemo.show_progress)
 
+        @ui.page("/shell")
+        async def show_shell_demo(client: Client):
+            return await self.page(client, NiceGuiWidgetsDemo.show_shell_demo)
+
+        @ui.page("/gpxviewer")
+        async def show_gpxviewer(client: Client):
+            return await self.page(client, NiceGuiWidgetsDemo.show_gpxviewer)
+
         @ui.page("/langs")
         async def show_langs(client: Client):
             return await self.page(client, NiceGuiWidgetsDemo.show_langs)
 
+        @ui.page("/leaflet")
+        async def show_leaflet(client: Client):
+            return await self.page(client, NiceGuiWidgetsDemo.show_leaflet)
+
+        @ui.page("/log_handler")
+        async def show_log_handler_demo(client: Client):
+            return await self.page(
+                client, NiceGuiWidgetsDemo.show_log_element_handler_demo
+            )
+
         @ui.page("/color_schema")
         async def show_color_schema(client: Client):
             return await self.page(client, NiceGuiWidgetsDemo.show_color_schema)
@@ -827,6 +1109,10 @@ class NiceGuiWidgetsDemoWebserver(InputWebserver):
         async def show_hide_show(client: Client):
             return await self.page(client, NiceGuiWidgetsDemo.show_hide_show_demo)
 
+        @ui.page("/taskrunner")
+        async def show_taskrunner_demo(client: Client):
+            return await self.page(client, NiceGuiWidgetsDemo.show_taskrunner_demo)
+
         @ui.page("/tristate")
         async def show_tristate_demo(client: Client):
             return await self.page(client, NiceGuiWidgetsDemo.show_tristate_demo)
