@@ -25,7 +25,7 @@ class TestDemoWebserver(WebserverTest):
         test API docs access
         """
         # self.debug=True
-        html = self.getHtml("/docs")
+        html = self.get_html("/docs")
         self.assertTrue("Swagger" in html)
 
     def testApi(self):
@@ -38,7 +38,7 @@ class TestDemoWebserver(WebserverTest):
         for path in paths:
             response = self.get_response(path)
             yaml_str = response.text
-            projects = Projects.from_yaml(yaml_str)
+            projects = Projects.from_yaml(yaml_str) # @UndefinedVariable
             if debug:
                 for project in projects.projects:
                     print(project)
